;******************** (C) COPYRIGHT 2005 STMicroelectronics ********************
;* File Name          : 91x_vect.s
;* Author             : MCD Application Team
;* Date First Issued  : 10/25/2005 :  Beta Version V0.1
;* Description        : This File used to initialize the exception and IRQ
;*                      vectors, and to enter/return to/from exceptions
;*                      handlers.
;*******************************************************************************
; History:
; 10/25/2005 :  Beta Version V0.1
;*******************************************************************************
; THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
; CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME. AS
; A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
; OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
; OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
; CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
;******************************************************************************/

#include "FreeRTOSConfig.h"
#include "ISR_Support.h"

        MODULE	?RESET
	COMMON	INTVEC:CODE(2)			
	CODE32
        EXPORT LINK

VectorAddress			EQU    0xFFFFF030  ; VIC Vector address register address.
VectorAddressDaisy		EQU    0xFC000030  ; Daisy VIC Vector address register
                                          ; address.
LINK		        	EQU    0x0

I_Bit				EQU    0x80 ; when I bit is set, IRQ is disabled
F_Bit				EQU    0x40 ; when F bit is set, FIQ is disabled
STR910_VIC0_BASE		EQU     0xFFFFF000		
;*******************************************************************************
;* Macro Name     : SaveContext
;* Description    : This macro is used to save the context before entering
;                   an exception handler.
;* Input          : The range of registers to store.
;* Output         : none
;*******************************************************************************

SaveContext MACRO reg1,reg2
        STMFD  sp!,{reg1-reg2,lr} ; Save The workspace plus the current return
                              ; address lr_ mode into the stack.
        MRS    r1,spsr        ; Save the spsr_mode into r1.
        STMFD  sp!,{r1}       ; Save spsr.
        ENDM

;*******************************************************************************
;* Macro Name     : RestoreContext
;* Description    : This macro is used to restore the context to return from
;                   an exception handler and continue the program execution.
;* Input          : The range of registers to restore.
;* Output         : none
;*******************************************************************************

RestoreContext MACRO reg1,reg2
        LDMFD   sp!,{r1}        ; Restore the saved spsr_mode into r1.
        MSR     spsr_cxsf,r1    ; Restore spsr_mode.
        LDMFD   sp!,{reg1-reg2,pc}^; Return to the instruction following...
                                ; ...the exception interrupt.
        ENDM


;*******************************************************************************
;            Import  the __program_start address from 91x_init.s
;*******************************************************************************

       	IMPORT  __program_start

;*******************************************************************************
;                      Import exception handlers
;*******************************************************************************

	IMPORT  vPortYieldProcessor		; FreeRTOS SWI handler

;*******************************************************************************
;            Export Peripherals IRQ handlers table address
;*******************************************************************************

;*******************************************************************************
;                        Exception vectors
;*******************************************************************************

        LDR     PC, Reset_Addr
        LDR     PC, Undefined_Addr
        LDR     PC, SWI_Addr
        LDR     PC, Prefetch_Addr
        LDR     PC, Abort_Addr
        NOP                             ; Reserved vector
        LDR    	PC, IRQ_Addr
        LDR     PC, FIQ_Addr

;*******************************************************************************
;               Exception handlers address table
;*******************************************************************************

Reset_Addr      DCD     __program_start
Undefined_Addr  DCD     UndefinedHandler
SWI_Addr        DCD     vPortYieldProcessor
Prefetch_Addr   DCD     PrefetchAbortHandler
Abort_Addr      DCD     DataAbortHandler
                DCD     0               ; Reserved vector
IRQ_Addr        DCD     IRQHandler
FIQ_Addr        DCD     FIQHandler


;*******************************************************************************
;                         Exception Handlers
;*******************************************************************************

; - NOTE -
; The IRQ and SWI handlers are the only managed exception.

UndefinedHandler
		b	UndefinedHandler

PrefetchAbortHandler
		b	PrefetchAbortHandler

DataAbortHandler
		b	DataAbortHandler

FIQHandler
		b	FIQHandler

DefaultISR
		b	DefaultISR


;*******************************************************************************
;* Function Name  : IRQHandler
;* Description    : This function called when IRQ exception is entered.
;* Input          : none
;* Output         : none
;*******************************************************************************

IRQHandler
	STMDB	SP!, {R0}	              ; push R0

        ldr	r0, =(STR910_VIC0_BASE)
	ldr	r0, [r0, #0x00]
        tst	r0, #(1<<0x08)
	bne	IRQHandler0
	tst	r0, #(1<<0x09)
	bne	IRQHandler0

	LDMIA	SP!, {R0}                     ; pop R0


	portSAVE_CONTEXT				; Save the context of the current task.

	LDR    r0, = VectorAddress
	LDR    r0, [r0]					; Read the routine address
	LDR    r1, = VectorAddressDaisy
	LDR    r1, [r1]
	MOV	   lr, pc
	bx	   r0
	LDR    r0, = VectorAddress			; Write to the VectorAddress to clear the
	STR    r0, [r0]					; respective interrupt in the internal interrupt
	LDR    r1, = VectorAddressDaisy		        ; Write to the VectorAddressDaisy to clear the
	STR    r1,[r1]					; respective interrupt in the internal interrupt
	
	portRESTORE_CONTEXT				; Restore the context of the selected task.


        B     IRQ_End


IRQHandler0
	LDMIA	SP!, {R0}                     ; pop R0
       SUB    lr,lr,#4        ; Update the link register

       SaveContext r0,r12     ; Save the workspace plus the current
                              ; return address lr_irq and spsr_irq
       LDR    r0, = VectorAddress
       LDR    r0, [r0]                   ; Read the routine address
       LDR    r1, = VectorAddressDaisy
       LDR    r1, [r1]
       ; Padding between the acknowledge and re-enable of interrupts
       ; For more details, please refer to the following URL
       ; http://www.arm.com/support/faqip/3682.html
       NOP
       NOP
       MSR    cpsr_c,#0x1F            ; Switch to SYS mode and enable IRQ
       STMFD  sp!,{lr}                ; Save the link register.
       LDR    lr, = IRQ_ReturnAddress ; Read the return address.
       MOV    pc, r0                  ; Branch to the IRQ handler.

IRQ_ReturnAddress
       LDMFD  sp!,{lr}                ; Restore the link register.
       MSR    cpsr_c,#0xD2 | I_Bit    ; Switch to IRQ mode and disable IRQ
       LDR    r0, = VectorAddress     ; Write to the VectorAddress to clear the
       STR    r0, [r0]                ; respective interrupt in the internal interrupt
       LDR    r1, = VectorAddressDaisy  ; Write to the VectorAddressDaisy to clear the
       STR    r1,[r1]                   ; respective interrupt in the internal interrupt

       RestoreContext r0,r12          ; Restore the context and return to the...
                                      ; ...program execution.
//       LDMIA	SP!, {R0}       ; pop R0

IRQ_End
      NOP

       LTORG

       END

;******************* (C) COPYRIGHT 2003 STMicroelectronics *****END OF FILE****
