/* Library includes. */
#include "91x_lib.h"

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"

/* Prototypes */
void Set_System(void);
void vReadThread(void * pvParameters);
void vWriteThread(void * pvParameters);

/* Global */
GPIO_InitTypeDef GPIO_InitStructure;
xTaskHandle xReadTask, xWriteTask;
u8 ucParameterToPass;
u8 u_port=0x0;
int ret;

int main()
{
  #ifdef DEBUG
	debug();
  #endif
  
  /* Configure the system */
  Set_System();
  
  /* Creation of the both tasks */
  ret = xTaskCreate( vReadThread, "ReadTask", configMINIMAL_STACK_SIZE, &ucParameterToPass, tskIDLE_PRIORITY, &xReadTask );
  ret = xTaskCreate( vWriteThread, "WriteTask", configMINIMAL_STACK_SIZE, &ucParameterToPass, tskIDLE_PRIORITY, &xWriteTask );
  
  /* Start the scheduling */
  vTaskStartScheduler();
  
  for( ;; );
}

void Set_System(void)
{
  /* Set Flash banks size & address */
  FMI_BankRemapConfig( 4, 2, 0, 0x80000 );
  
  /* FMI Waite States */
  FMI_Config( FMI_READ_WAIT_STATE_2, FMI_WRITE_WAIT_STATE_0, FMI_PWD_ENABLE, FMI_LVD_ENABLE, FMI_FREQ_HIGH );
  
  /* Configure the FPLL = 96MHz, and APB to 48MHz. */
  SCU_PCLKDivisorConfig( SCU_PCLK_Div2 );
  SCU_PLLFactorsConfig( 192, 25, 2 );
  SCU_PLLCmd( ENABLE );
  SCU_MCLKSourceConfig( SCU_MCLK_PLL );
  
  WDG_Cmd( DISABLE );
  VIC_DeInit();
  
  /* General purpose IO (GPIO) Initialisierung */
  SCU_APBPeriphClockConfig(__GPIO6 ,ENABLE);
  GPIO_DeInit(GPIO6); // Port 6 Pin 0-3 sind LEDs
  GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
  GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
  GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
  GPIO_Init(GPIO6, &GPIO_InitStructure);
  
  SCU_APBPeriphClockConfig(__GPIO7 ,ENABLE);
  GPIO_DeInit(GPIO7); // Port 7 Pin 0-3 sind Taster
  GPIO_InitStructure.GPIO_Direction = GPIO_PinInput;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
  GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
  //GPIO_InitStructure.GPIO_IPConnected = GPIO_IPConnected_Enable;
  GPIO_InitStructure.GPIO_Alternate = GPIO_InputAlt1;
  GPIO_Init(GPIO7, &GPIO_InitStructure);
  
  /* Enable VIC clock */
  SCU_AHBPeriphClockConfig(__VIC, ENABLE);
  SCU_AHBPeriphReset(__VIC, DISABLE);
}

void vReadThread(void * pvParameters)
{
  for( ; ; )
  {
    u_port = GPIO_Read(GPIO7);
  }
}

void vWriteThread(void * pvParameters)
{
  for( ; ; )
  {
    GPIO_Write(GPIO6,u_port);
  }
}
